/* Copyright Nicolas GUEDON - February 2016 - FabLab Lannion Ker Nel */

#include <IRremote.h>
#include <EEPROM.h>
#include <Servo.h> 

// Declaration variable reception infrarouge
 int IR_PIN = 3; //Pin de la reception infrarouge
 IRrecv irrecv(IR_PIN);
 decode_results IRCODE; //C'est le code reçu venant de la telecommande
 unsigned long codeRecu = 0;
// Les codes et fonctions associes (code par défaut mais sont configurable sans ecriture soft)
 unsigned long codeWake = 11111111;
 unsigned long codeShut = 22222222;

// Variable utiles pour les controles de boucles
int loopWakeStop = 1;
int loopShutStop = 1;

// Declaration pour la led de signalisation
int ledpin = 7;

// Declaration pour le switch d'apprentissage
int switchMaster = 8;
int switchSlave = 9;

// Declaration Servo
int servoVCC = 10;
int servoCommand = 5;
int angleOrig = 160;
int angleCible = 95;
Servo myservo;

void setup() {
  EEPROM.get(100, codeWake); // On recupere le code telecommande de WOl situee en EEPROM
  EEPROM.get(500, codeShut); // On recupere le code telecommande de Shutdown situee en EEPROM
 
  Serial.begin(9600); // Port serie pour debug
  Serial.println("codeWake interne:");
  Serial.println(codeWake);
  Serial.println("codeShut interne:");
  Serial.println(codeShut);
  //setup Servo
  pinMode(servoVCC, OUTPUT);
  digitalWrite(servoVCC, LOW);
  pinMode(servoCommand, OUTPUT);
  myservo.attach(servoCommand);
  myservo.write(angleOrig);

  //setup Led signalisation
  pinMode(ledpin, OUTPUT);
  digitalWrite(ledpin, LOW);

  //setup switch
  pinMode(switchMaster, OUTPUT);
  digitalWrite(switchMaster, LOW);
  pinMode(switchSlave, INPUT_PULLUP);

  // Start the infrared receiver
  irrecv.enableIRIn();
  irrecv.blink13(true);
  //Clignottement de la led de signalisation pour indiquer demarrage ok
  digitalWrite(ledpin, HIGH);
  delay(200);
  digitalWrite(ledpin, LOW);

}

void loop() {
  //IR receiving part
  if (irrecv.decode(&IRCODE)) {
    Serial.print("Receive decode");
    Serial.println(IRCODE.value);
    codeRecu = IRCODE.value;
    if(IRCODE.value != 4294967295){
      digitalWrite(ledpin, HIGH);
      delay(600);
      digitalWrite(ledpin, LOW);
    }
    Serial.println("TEST");
    irrecv.resume(); // Receive the next value
  }
  delay(30);
  // End of IR receiving part

  // Switch to change IR code stored
  if(digitalRead(switchSlave) == LOW) {
    // Pour le code Wake
    Serial.println("Wait for the new wake IR code after 5 blink");
    blinker(5, 200); // Clignottement de la led 5 fois avec espacement de 200ms
    int i = 0; // To control timeout
    digitalWrite(ledpin, HIGH); // On allume la led pour attendre le code
    while(loopWakeStop) {
      if (irrecv.decode(&IRCODE)) {
        Serial.print("Ancien code: ");
        Serial.println(codeWake);
        codeWake = IRCODE.value;
        EEPROM.put(100,codeWake);
        loopWakeStop = 0;
        delay(100);
        Serial.print("Nouveau code: ");
        Serial.println(codeWake);
        delay(300);
        irrecv.resume();
      }
       if(i == 4000) { //Timeout if no code is received (20s environ)
       loopWakeStop = 0;
      }
      i++;
      delay(10);
    }
    loopWakeStop = 1;
    // Fin pour le code Wake
    digitalWrite(ledpin, LOW);
    delay(1000);
    // Pour le code shut
    Serial.println("Wait for the new shut IR code after 10 blink");
    blinker(10, 200); // Clignottement de la led 10 fois avec espacement de 200ms
    i = 0; // To control timeout
    digitalWrite(ledpin, HIGH);
    while(loopShutStop) {
      if (irrecv.decode(&IRCODE)) {
        Serial.print("Ancien code: ");
        Serial.println(codeShut);
        codeShut = IRCODE.value;
        EEPROM.put(500,codeShut);
        loopShutStop = 0;
        delay(100);
        Serial.print("Nouveau code: ");
        Serial.println(codeShut);
        delay(300);
        irrecv.resume();
      }
       if(i == 4000) { //Timeout if no code is received (20s environ)
       loopShutStop = 0;
      }
      i++;
      delay(10);
    }
    loopShutStop = 1;
    // Fin pour le code Shut
    digitalWrite(ledpin, LOW);
    delay(1000);
  }
  // Action to do link to IRcode
  if(codeRecu == codeWake) { //Action Wake
    codeRecu = 0; // On reinitialise le code
    Serial.println("Wake up!!!");
    servoAction(300);
  } else if (codeRecu == codeShut) { // Action force shutdown
    codeRecu = 0;
    Serial.println("Time to shutdown!!!");
    servoAction(5000);
  } else { 
    codeRecu = 0; // On reinitialise le code
    //Autre cas, on ne fait rien
  }
}


void servoAction(int duree) {
  digitalWrite(servoVCC, HIGH);
  delay(500);
  myservo.write(angleCible);
  delay(duree);
  myservo.write(angleOrig);
  delay(200);
  delay(200);
  digitalWrite(servoVCC, LOW);
}


// Fonction pour faire clignotter la led x nombre de foi (x max = 100) avec un delai de z (z max = 8000)
void blinker(int x, int z) {
  if (z > 8000) {
    z = 8000;
  }
  if (x > 100) {
    x = 100;
  }
  for(int j=0; j<x; j++) {
      digitalWrite(ledpin, HIGH);
      delay(z);
      digitalWrite(ledpin,LOW);
      delay(z);
    }
  
}

  
